package com.perforce.hws.core.p4base;

import com.perforce.p4java.exception.AccessException;
import com.perforce.p4java.exception.ConfigException;
import com.perforce.p4java.exception.ConnectionException;
import com.perforce.p4java.exception.RequestException;
import com.perforce.p4java.server.IServer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Supplier;

/**
 * Helpers around logging into the server and obtaining tickets.
 */
public interface AuthMethods extends P4Methods {

    /**
     * Logs into the underlying server.
     * <p>
     * If you have no password, do not call this method.
     *
     * @param server   The connection
     * @param user     Perforce login
     * @param password User's password.
     * @param allHosts The "-a" flag for the p4 login command.
     * @return The authTicket (should be a valid p4 ticket)
     * @throws AccessException When the server has denied access (i.e., invalid password.)
     */
    default String logIn(Supplier<IServer> server,
                         String user,
                         String password,
                         boolean allHosts)
            throws AccessException {

        String ticket = null;

        try {
            IServer s = server.get();
            if (s.isConnected() &&
                    (s.getUserName() != null && !s.getUserName().equals(user))) {
                s.disconnect();
            }

            s.setUserName(user);

            if (!s.isConnected()) {
                s.connect();
            }

            Map<String, Object> input = new HashMap<>();
            input.put("password", password);

            List<String> args = new ArrayList<>();
            args.add("-p");
            if (allHosts) {
                args.add("-a");
            }
            String[] params = args.toArray(new String[0]);

            execInput(server, input, "login", params);

            // Yep. We don't actually get response data from P4Java. We have
            // to pull this value from it's internal cache.
            ticket = s.getAuthTicket();

        } catch (ConnectionException | RequestException | AccessException | ConfigException e) {
            throw new P4JavaRuntimeException(e);
        } catch (ServerErrorException e) {
            List<ErrorId> errors = e.getResultMap().getErrors();
            if (errors.contains(ErrorIds.MsgServer_NoSuchUser) ||
                    errors.contains(ErrorIds.MsgServer_BadPassword) ||
                    errors.contains(ErrorIds.MsgServer_BadPassword0) ||
                    errors.contains(ErrorIds.MsgServer_BadPassword1) ||
                    errors.stream().anyMatch(eid -> "805445297".equals(eid.toString()))) {
                throw new AuthError(e);
            } else {
                throw e;
            }
        }

        return ticket;
    }

}
